@@ -1,7 +1,9 @@
-from typing import List, Optional, Type, Union
+import copy
+from typing import List, Optional, Type
 
 import jsonref
 from django.db.models.fields import Field
+from jsonschema_to_openapi.convert import convert as convert_to_openapi
 from pydantic.error_wrappers import ValidationError as PydanticValidationError
 from pydantic.main import BaseModel
 from rest_framework.serializers import JSONField
@@ -11,7 +13,9 @@ __all__ = ["PydanticJsonFieldSerializer"]
 
 class PydanticJsonFieldSerializer(JSONField, Field):
     """
-    Serializer for serializing our custom PydanticJsonField
+    Serializer for serializing our custom PydanticJsonField.
+
+    Provides annotations for both drf-spectacular and drf-yasg
     """
 
     class Meta:
@@ -22,7 +26,15 @@ class PydanticJsonFieldSerializer(JSONField, Field):
     ):
         super().__init__(*args, **kwargs)
 
-        self.__set_information_for_drf_yasg(pydantic_models)
+        self.schema = self.__schema_information(pydantic_models)
+
+        # Set schema for drf-spectacular
+        self.coreapi_schema = convert_to_openapi(
+            copy.deepcopy(self.__schema_information(pydantic_models))
+        )
+
+        # Set schema for drf-yasg
+        PydanticJsonFieldSerializer.Meta.swagger_schema_fields = self.schema
 
         self.pydantic_models = pydantic_models if pydantic_models else []
 
@@ -53,13 +65,15 @@ class PydanticJsonFieldSerializer(JSONField, Field):
     def __schema_field_from_pydantic(self, pydantic_model: Type[BaseModel]) -> dict:
         return jsonref.loads(pydantic_model.schema_json())
 
-    def __set_information_for_drf_yasg(
+    def __schema_information(
         self, pydantic_models: Optional[List[Type[BaseModel]]]
-    ):
-        """Set information for swagger generator drf_yasg"""
+    ) -> dict:
+        """
+        Returns a JSON schema that is used for representing the potential values of this field
+        """
 
         if pydantic_models is None or pydantic_models.__len__() == 0:
-            schema_information = {}
+            schema_information = {"type": "object"}
 
         elif pydantic_models.__len__() > 1:
             schema_information = {
@@ -71,4 +85,17 @@ class PydanticJsonFieldSerializer(JSONField, Field):
         else:
             schema_information = self.__schema_field_from_pydantic(pydantic_models[0])
 
-        PydanticJsonFieldSerializer.Meta.swagger_schema_fields = schema_information
+        return schema_information
+
+
+try:
+    from drf_spectacular.extensions import OpenApiSerializerFieldExtension
+
+    class PydanticJsonFieldSerializerExtension(OpenApiSerializerFieldExtension):
+        target_class = PydanticJsonFieldSerializer
+
+        def map_serializer_field(self, auto_schema, direction):
+            return self.target.schema or {"type": "object"}
+
+except Exception:
+    pass
